#define LILYGO_T5_V213

#include <GxEPD.h>
#include <GxDEPG0213BN/GxDEPG0213BN.h>
#include <Fonts/FreeMonoBold9pt7b.h>
#include <Fonts/FreeMono9pt7b.h>
#include <Fonts/Org_01.h>
#include <GxIO/GxIO_SPI/GxIO_SPI.h>
#include <GxIO/GxIO.h>
#include <WiFi.h>
#include <WiFiMulti.h>
#include <HTTPClient.h>
#include <ArduinoJson.h> // Ajout de la bibliothèque ArduinoJson pour analyser les données JSON

#include <NTPClient.h>
#include <WiFiUdp.h>

#include <esp_wifi.h>
#include <esp_system.h>
#include <esp_event.h>
#include <esp_sleep.h>

WiFiMulti wifiMulti;


#define EPD_MOSI                (23)
#define EPD_MISO                (-1)
#define EPD_SCLK                (18)
#define EPD_CS                  (5)

#define EPD_BUSY                (4)
#define EPD_RSET                (16)
#define EPD_DC                  (17)

#define BUTTON_1                (39)
#define BUTTONS                 {39}

#define BUTTON_COUNT            (1)

#define LED_PIN                 (19)
#define LED_ON                  (LOW)

#define ADC_PIN                 (35)

#define _HAS_ADC_DETECTED_
#define _HAS_LED_
#define _HAS_SDCARD_

int counter = 0;

float vbat = 0;

GxIO_Class io(SPI,  EPD_CS, EPD_DC,  EPD_RSET);
GxEPD_Class display(io, EPD_RSET, EPD_BUSY);


const long GMT_offset_sec = -4 * 3600; // GMT-4 
const int daylightOffset_sec = 0;

WiFiUDP ntpUDP;
NTPClient timeClient(ntpUDP, "europe.pool.ntp.org", GMT_offset_sec);

const char *api_key = "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"; // API Key OpenWeatherMap
const char *city = "Gustavia"; // Nom de la ville
const char *country = "BL"; // Code pays
const char *language = "fr"; // Langue
const char *units = "metric"; // Unités (Métrique)
const char *hemisphere = "north"; // Hémisphère

String mapWindDirection(int degrees) {
    const char *directions[] = {"N", "NNE", "NE", "ENE", "E", "ESE", "SE", "SSE", "S", "SSO", "SO", "OSO", "O", "ONO", "NO", "NNO"};
    int index = int(round(((float)degrees / 22.5) + 0.5)) % 16; // Conversion en entier avant d'utiliser l'opérateur modulo
    return directions[index];
}

void setup(void)
{
   
    Serial.begin(115200);
    Serial.println();
    pinMode(LED_PIN, OUTPUT);
    delay(500);
    digitalWrite(LED_PIN, LOW);
    delay(500);
    digitalWrite(LED_PIN, HIGH);
    delay(500);
    digitalWrite(LED_PIN, LOW);
    
    vbat = map(analogRead(35), 0, 4095, 100, 7200)/1000.00;   
    Serial.print("Batterie: ");
    Serial.println(vbat);
    if (vbat <= 3.5) goToSleep();
    delay(500);
    digitalWrite(LED_PIN, HIGH);
    delay(500);
    digitalWrite(LED_PIN, LOW);
    delay(500);
    digitalWrite(LED_PIN, HIGH);
    Serial.begin(115200);
    Serial.println();
    Serial.println("setup");

    SPI.begin(EPD_SCLK, EPD_MISO, EPD_MOSI);
    display.init();

    // Connexion au réseau WiFi
    WiFi.mode(WIFI_STA);
    wifiMulti.addAP("Wifi SSID 1", "Wifi PASSWORD 1");
    wifiMulti.addAP("Wifi SSID 2", "Wifi PASSWORD 2");
    wifiMulti.addAP("Wifi SSID 3", "Wifi PASSWORD 3");

    Serial.println("Connecting to WiFi..");   
    counter = 0;
    while (wifiMulti.run() != WL_CONNECTED) {
      delay(500);
      Serial.print(".");
      counter++;
      if (counter >= 10) goToSleep();
    }
    Serial.println("Connected to WiFi");

    Serial.println("setup done");
}

void loop()
{
    display.setRotation(3);
    display.fillScreen(GxEPD_BLACK);
    display.setTextColor(GxEPD_WHITE);
    display.setFont(&FreeMonoBold9pt7b);
    display.setCursor(5, 17);

    
    // Obtenir l'heure actuelle à partir du serveur NTP
    timeClient.update();
    
   // Obtenir le temps au format UNIX Epoch
    unsigned long epochTime = timeClient.getEpochTime();

    // Convertir le temps en une structure tm représentant la date et l'heure locales
    struct tm *timeinfo;
    timeinfo = localtime((const time_t *)&epochTime);

    // Formater la date avec des numéros sur deux chiffres
    char formattedDate[11];
    sprintf(formattedDate, "%02d/%02d/%d", timeinfo->tm_mday, timeinfo->tm_mon + 1, timeinfo->tm_year + 1900);
    String currentDate = String(formattedDate);

    // Formater l'heure avec des numéros sur deux chiffres
    char formattedTime[6];
    sprintf(formattedTime, "%02d:%02d", timeinfo->tm_hour, timeinfo->tm_min);
    String currentTime = String(formattedTime);
    
    // Effectuer une requête HTTP pour récupérer les données météo depuis OpenWeatherMap
    HTTPClient http;
    String url = "http://api.openweathermap.org/data/2.5/weather?q=" + String(city) + "," + String(country) + "&lang=" + String(language) + "&units=" + String(units) + "&appid=" + String(api_key);
    http.begin(url);
    Serial.println("URL: ");
    Serial.println(url);
    int httpCode = http.GET();

    if (httpCode > 0) {
        String payload = http.getString();

        Serial.println("Recived JSON: ");
        Serial.println(payload);
        Serial.println();

        // Analyser les données JSON de la réponse de l'API
        DynamicJsonDocument doc(1024); // Taille du document JSON
        DeserializationError error = deserializeJson(doc, payload);

        if (error) {
            Serial.print("deserializeJson() failed: ");
            Serial.println(error.c_str());
            return;
        }

        // Récupérer les informations météo spécifiques du document JSON
        float temperature = doc["main"]["temp"];
        int humidity = doc["main"]["humidity"];
        float wind_speed = doc["wind"]["speed"];
        int wind_direction = doc["wind"]["deg"];
        float wind_gusts = doc["wind"]["gust"];
        int pressure = doc["main"]["pressure"];
        int rain = doc["rain"]["1h"]; // Pluviométrie (si disponible)

        // Afficher les informations météo sur l'écran e-paper
        
        display.setFont(&FreeMonoBold9pt7b);
        display.setCursor(5, 15);
        display.println("Temperature: " + String((float)temperature) + "°C");
        display.setCursor(5, 15 + (17 * 1));
        display.println("Humidite: " + String(humidity) + "%");
        display.setCursor(5, 15 + (17 * 2));
        display.println("Vent: " + String(((float)wind_speed * 3.6 )) + " Km/h");
        display.setCursor(5, 15 + (17 * 3));
        //display.println("Direction: " + String(wind_direction) + "°");
        display.println("Direction: " + mapWindDirection(wind_direction));
        display.setCursor(5, 15 + (17 * 4));
        display.println("Rafales: " + String(((float)wind_gusts * 3.6 )) + " Km/h");
        display.setCursor(5, 15 + (17 * 5));
        display.println("Pressure: " + String(pressure) + " hPa");
        display.setCursor(5, 15 + (17 * 6));
        display.println("PLuie 1h: " + String((float)rain) + " mm");

        
        display.setRotation(2);
        display.setFont(&Org_01);
        display.setCursor(10, 240);
        display.println(currentTime);
        display.setCursor(10, 246);
        display.println(currentDate);

        vbat = map(analogRead(35), 0, 4095, 100, 7200)/1000.00;
        
        display.setCursor(100, 240);
        display.println("Batt:");
        display.setCursor(100, 246);
        display.println(String(vbat) + "V");
        
    } else {
        Serial.println("Error on HTTP request");
    }

    http.end();

    display.update();
    digitalWrite(LED_PIN, LOW);
    delay(1000);
    goToSleep();
}

void goToSleep(){
// Mise en veille pendant 5 minutes
    esp_sleep_enable_timer_wakeup(15 * 60 * 1000000); // 5 minutes en microsecondes
    delay(1000);
    esp_deep_sleep_start();
    delay(5000);
}